@@ -40,13 +40,13 @@ An abstract VHDL language model.
 :copyright: Copyright 2007-2021 Patrick Lehmann - Bötzingen, Germany
 :license: Apache License, Version 2.0
 """
-from enum import IntEnum, unique, Enum
-from typing import List
+from enum     import IntEnum, unique, Enum
+from typing import List, Iterable
 
-from pydecor import export
+from pydecor  import export
 
 
-__version__ = "0.11.3"
+__version__ = "0.11.4"
 
 
 @export
@@ -262,11 +262,18 @@ class DesignUnit(ModelEntity, NamedEntity):
 	"""
 	A ``DesignUnit`` is a base-class for all design units.
 	"""
+	_contextItems: List['Context']
 
-	def __init__(self, identifier: str):
+	def __init__(self, identifier: str, contextItems: Iterable['Context'] = None):
 		super().__init__()
 		NamedEntity.__init__(self, identifier)
 
+		self._contextItems = [] if contextItems is None else [c for c in contextItems]
+
+	@property
+	def ContextItems(self) -> List['Context']:
+		return self._contextItems
+
 
 @export
 class PrimaryUnit(DesignUnit):
